/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import java.util.List;
import java.util.Map;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;

import org.dive4elements.river.artifacts.model.minfo.BedDiffYearResult;
import org.dive4elements.river.model.BedHeightSingleValue;
import org.dive4elements.river.artifacts.model.minfo.BedHeightSingleData;

import org.dive4elements.river.artifacts.model.minfo.MorphologicWidth;

public class BedWidthProcessor extends DefaultProcessor {

    private final static Logger logger =
            Logger.getLogger(BedWidthProcessor.class);

    public static final String I18N_AXIS_LABEL_DEFAULT =
        "Breite [m]";
    public static final String I18N_AXIS_LABEL =
        "chart.beddifference.yaxis.label.morph";

    protected String yAxisLabel;
    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        CallContext context = generator.getCallContext();
        Map<String, String> metaData = bundle.getFacet().getMetaData();
        StyledXYSeries series = new StyledXYSeries(bundle.getFacetDescription(),
                theme);
        series.putMetaData(metaData, bundle.getArtifact(), context);
        yAxisLabel = metaData.get("Y");
        Object data = bundle.getData(context);
        String facetName = bundle.getFacetName();

        if (facetName.equals(FacetTypes.BED_DIFFERENCE_SOUNDING_WIDTH) &&
            data instanceof BedDiffYearResult) {
            BedDiffYearResult bData = (BedDiffYearResult) data;
            StyledSeriesBuilder.addPoints(
                series,
                bData.getSoundingWidthData(),
                false,
                0.110d);
        } else if (facetName.equals(FacetTypes.BED_DIFFERENCE_MORPH_WIDTH1) &&
            data instanceof BedDiffYearResult) {
            BedDiffYearResult bData = (BedDiffYearResult) data;
            StyledSeriesBuilder.addPoints(series, bData.getMorphWidth1Data(), true);
        } else if (facetName.equals(FacetTypes.BED_DIFFERENCE_MORPH_WIDTH2) &&
            data instanceof BedDiffYearResult) {
            BedDiffYearResult bData = (BedDiffYearResult) data;
            StyledSeriesBuilder.addPoints(series, bData.getMorphWidth2Data(), true);
        } else if (data instanceof MorphologicWidth) {
            MorphologicWidth bData = (MorphologicWidth) data;
            StyledSeriesBuilder.addPoints(series, bData.getAsArray(), true);
        } else if (data instanceof BedHeightSingleData) {
            BedHeightSingleData bData = (BedHeightSingleData)data;
            double[] width = bData.getSoundingWidths();
            double[] stations = bData.getStations().toNativeArray();

            for (int i = 0; i < width.length; i++) {
                series.add(stations[i], width[i], false);
            }
        } else if (data instanceof List<?>) {
            List<BedHeightSingleValue> bData = (List<BedHeightSingleValue>)data;

            for(BedHeightSingleValue bvalue: bData) {
                series.add(bvalue.getStation(), bvalue.getSoundingWidth());
            }
        } else {
            logger.error("Unknown data for facet: " + bundle.getFacetName());
        }

        generator.addAxisSeries(series, axisName, visible);
    }

    @Override
    public boolean canHandle(String facettype) {
        return facettype.equals(FacetTypes.BED_DIFFERENCE_SOUNDING_WIDTH) ||
            facettype.equals(FacetTypes.BED_DIFFERENCE_MORPH_WIDTH1) ||
            facettype.equals(FacetTypes.BED_DIFFERENCE_MORPH_WIDTH2) ||
            facettype.equals(FacetTypes.MORPHOLOGIC_WIDTH) ||
            facettype.equals(FacetTypes.BEDHEIGHT_SOUNDING_WIDTH);
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        if (yAxisLabel != null && !yAxisLabel.isEmpty()) {
            return generator.msg(yAxisLabel, I18N_AXIS_LABEL_DEFAULT);
        }
        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT);
    }
}
